
;    .module dma
;    .export dma_go
;    .export dma_wait
;    .export dma_finished
;    .export mdma_finished
;    .export odma_finished
;    .export dma_write
;    .export dma_read

;    .export dmastuff    
;    .export dma_count    
;    .export dma_len    
;    .export dma_base    
;    .export dma_xpos    
;    .export dma_ypos    

; dmastuff.  Various useful DMA routines.
;
; dma.s - useful DMA thangs.
;
; this version (20/11/97) is hacked for "paranoid" DMA - that is, dma_go
; actually waits for dma_finished, before and after the DMA it sets up.
; This is coz DMA has changed a lot since I wrote much of my stuff
; so I am running with all DMA optimization disallowed until I get the time
; to optimise for new-stylee DMA.
;
;
; Uses a common DMA-info structure that I use a lot in my graphics code.
; This is a single vector that I refer to as dmastuff, with the following format:
;
; dma_xpos = Current X address for dma writes.  16:16
; dma_ypos = Current Y address for dma writes.  16:16
; dma_len = Length of the DMA transfer.  Integer.
; dma_base = The internal address of the DMA transfer.
;
; the DMA_GO routine also expects the existence of a DMA command buffer in local ram, called
; dma__cmd, and assumes that you have already placed the external base address of the
; DMA transfer in the second long of that buffer (since it doesn't get changed as much as
; the rest).

; Here's where I usually define the dma stuff to be:

//	.segment sprited
//saveR1:	.ds.s	1
//saveR2:	.ds.s	1
	
	dmastuff	= v3
	dma_count	= r14
	dma_len		= r14
//	dma_base	= r15
	dma_xpos	= r12
	dma_ypos	= r13

	DMA_FINISHED_FLAG	= (1<<31)
	DMA_WAIT_FLAG		= (1<<30)

	dmactl_r0	= r0
	perfPtr_r1	= r1
	perfCount_r2	= r2

	.segment spritec
mdma_wait:
.if (WAITCNT_ENABLE == 1)
	add	#DMAWAITCNT_START,perfPtr_r1	; add base
	ld_s	(perfPtr_r1),perfCount_r2	; get current count
.endif
`loop:
	ld_s	mdmactl,dmactl_r0		; read MDMACTL
.if (WAITCNT_ENABLE == 1)
	add	#1,perfCount_r2			; increment performance counter
.else
	nop
.endif
	btst	#4,dmactl_r0			; is the bit clear?
{	bra	ne,`loop,nop			; nope, loop back
.if (WAITCNT_ENABLE == 1)
	or	#DMA_WAIT_FLAG,perfCount_r2	; set bit so it's easier to tell what this is
.endif
}
.if (WAITCNT_ENABLE == 1)
	sub	#1,perfCount_r2			; went one too far
.endif
{	rts	nop
.if (WAITCNT_ENABLE == 1)
;	st_s	perfCount_r2,(perfPtr_r1)	; save new count value
.endif
}
	

odma_wait:
`loop:
	ld_s	odmactl,r0
	nop
	btst	#4,r0
	bra	ne,`loop,nop
	rts	nop				
	
odma_finished:
`loop:
	ld_s	odmactl,r0
	nop
	bits	#4,>>#0,r0
	bra	ne,`loop,nop
	rts	nop

mdma_finished:
.if (WAITCNT_ENABLE == 1)
	add	#DMAWAITCNT_START,perfPtr_r1	; add base
	ld_s	(perfPtr_r1),perfCount_r2	; get current count
.endif
`loop:
	ld_s	mdmactl,r0
.if (WAITCNT_ENABLE == 1)
	add	#1,perfCount_r2			; increment performance counter
.else
	nop
.endif
	bits	#4,>>#0,r0
{	bra	ne,`loop,nop
.if (WAITCNT_ENABLE == 1)
	or	#DMA_FINISHED_FLAG,perfCount_r2	; set bit so it's easier to tell what this is
.endif
}
.if (WAITCNT_ENABLE == 1)
	sub	#1,perfCount_r2			; went one too far
.endif
{	rts	nop
.if (WAITCNT_ENABLE == 1)
	st_s	perfCount_r2,(perfPtr_r1)	; save new count value
.endif
}

mdma_wr_finished:
.if (WAITCNT_ENABLE == 1)
	add	#DMAWAITCNT_START,perfPtr_r1	; add base
	ld_s	(perfPtr_r1),perfCount_r2	; get current count
.endif
`loop:
	ld_s	mdmactl,r0
.if (WAITCNT_ENABLE == 1)
	add	#1,perfCount_r2			; increment performance counter
.else
	nop
.endif
//	bits	#4,>>#0,r0
//	bra	eq,`done
	bits	#7,>>#24,r0
{	bra	eq,`loop,nop
.if (WAITCNT_ENABLE == 1)
	or	#DMA_FINISHED_FLAG,perfCount_r2	; set bit so it's easier to tell what this is
.endif
}
`done:	
.if (WAITCNT_ENABLE == 1)
{	st_s	#(1<<11),mdmactl		; decrement done_cnt_wr
	sub	#1,perfCount_r2			; went one too far
}
{	rts	nop
	st_s	perfCount_r2,(perfPtr_r1)	; save new count value
}
.else
{	rts	nop
	st_s	#(1<<11),mdmactl		; decrement done_cnt_wr
}
.endif

mdma_rd_finished:
.if (WAITCNT_ENABLE == 1)
	add	#DMAWAITCNT_START,perfPtr_r1	; add base
	ld_s	(perfPtr_r1),perfCount_r2	; get current count
.endif
`loop:
	ld_s	mdmactl,r0
.if (WAITCNT_ENABLE == 1)
	add	#1,perfCount_r2			; increment performance counter
.else
	nop
.endif
//	bits	#4,>>#0,r0
//	bra	eq,`done
	bits	#7,>>#16,r0
{	bra	eq,`loop,nop
.if (WAITCNT_ENABLE == 1)
	or	#DMA_FINISHED_FLAG,perfCount_r2	; set bit so it's easier to tell what this is
.endif
}
`done:
.if (WAITCNT_ENABLE == 1)
{	st_s	#(1<<10),mdmactl		; decrement done_cnt_rd
	sub	#1,perfCount_r2			; went one too far
}
{	rts	nop
	st_s	perfCount_r2,(perfPtr_r1)	; save new count value
}
.else
{	rts	nop
	st_s	#(1<<10),mdmactl		; decrement done_cnt_rd
}
.endif

.if (0)
dma_write:

; This routine and dma_read are useful for grabbing blocks of longwords.
;
; Write r0 longs from internal address (r2) to external address (r1)
;
; Assumes that you have already done dma pending/terminated checking!

	push	v1
	push	v0,rz

	mv_s	#_bbdma,r5
	mv_s	#mdma_wait,r6
	btst	#31,r1          ;xram address on other bus?
	bra	eq,mainbus2,nop
	mv_s	#_obbdma,r5
	mv_s	#odma_wait,r6
mainbus2:


	mv_s	r0,r3       ;preserve r0
dod0:
	mv_s	#64,r0
	sub	r0,r3
	bra	ge,dod1,nop
	add	r3,r0
dod1:
;	jsr	_bbdma            ;launch the dma
	jsr	(r5)
;	push	v1
	lsl	#16,r0
	nop
    
;	jsr	dma_wait        ;wait until ready for new cmd
;	mv_s	#dma_wait,r11
	jsr	(r6)
	add	#256,r1
	add	#256,r2
;	pop	v1
	cmp	#0,r3
	bra	gt,dod0,nop
	pop	v0,rz
	pop	v1
	rts	t,nop
.endif

dma_read:

; Read r0 longs  from external ram address (r1) to internal ram address (r2).
;
; Assumes that you have already done dma pending/terminated checking!
;
; *** MODIFIED to use Other Bus and Really External Ram. ***

	push	v1
	push	v0,rz

	mv_s	#_bbdma,r5
	mv_s	#mdma_wait,r6
	btst	#31,r1          ;xram address on other bus?
	bra	eq,mainbus,nop
	mv_s	#_obbdma,r5
	mv_s	#odma_wait,r6
mainbus:
;	ld_s	object,r31
;	mv_s	#1,r31


	mv_s	r0,r3
dod2:
	mv_s	#64,r0
	sub	r0,r3
	bra	ge,dod3,nop
	add	r3,r0
dod3:
	jsr	(r5)		
{
;	push	v1			
	lsl	#16,r0
}
	bset	#13,r0			;read longs command
;	jsr	dma_wait        ;wait until ready for new cmd
	jsr	(r6)
	add	#256,r1
	add	#256,r2
;	pop	v1
	cmp	#0,r3
	bra	gt,dod2,nop
	pop	v0,rz
	pop	v1
	rts	nop
	
_bbdma:
	nop
								  
	mv_s	#dma__cmd,r4
	st_v	v0,(r4)			;cmd for read longs
	st_s	r4,mdmacptr
	rts	t,nop
;	pop	v1					;NOTE that this pop is still happening the first tick after we get back!

_obbdma:
	nop
	
	mv_s	#dma__cmd,r4
	st_v	v0,(r4)			;cmd for read longs
	st_s	r4,odmacptr
	rts	t,nop
;	pop	v1					;NOTE that this pop is still happening the first tick after we get back!
ramallama:
