;;;
;;; Matrix preparation
;;; 
;;; This is done once per sprite to set up the matrices that will transform a given YCrCb pixel
;;; into RGB, applying tint and translucency along the way.  Another matrix transforms from RGB
;;; to YCrCb.
;;;
	;;===================
	bright_v0	= v0
	bright_r_r0	= r0
	bright_g_r1	= r1
	bright_b_r2	= r2

	trans_fg_r0	= r0
	trans_bg_r1	= r1
	;;===================
matPrep:
	mv_s	#(4<<20),r0
	st_s	r0,linpixctl
	st_s	r0,xyctl
	st_s	#((4<<20)|(cachesize<<0)),uvctl
	
	push	v1
	push	v2
	push	v3
	push	v4
	push	v5
	push	v6
	push	v7
	
	;; 
	;; Apply translucency to background conversion
	;;
	ld_v	convToR,v4
	ld_v	convToG,v5
	ld_v	convToB,v6
	
.if (1)
;	ld_s	spr_trans,trans_bg_r1 
;	nop
;	and	#$FFFF0000,trans_bg_r1
;	asr	#6,trans_bg_r1		; turn 2.30 into 8.24 with sign intact
	ld_s	sprTransBG,trans_bg_r1
	nop
	
	mul_sv	trans_bg_r1,v4,>>#24,v4	; apply the translucency to the background matrix
	mul_sv	trans_bg_r1,v5,>>#24,v5
	mul_sv	trans_bg_r1,v6,>>#24,v6
.endif
	
	st_v	v4,convYToBG
	st_v	v5,convCrToBG
	st_v	v6,convCbToBG

	
	;; 
	;; Apply RGB brightness to the foreground matrix
	;; 
	ld_p	spr_bright,bright_v0	; get brightness scales
	nop
	
	lsr	#5,bright_r_r0
	lsr	#5,bright_g_r1
	lsr	#5,bright_b_r2

	ld_v	convToR,v4		; read the YCrCb->RGB matrices
	ld_v	convToG,v5
	ld_v	convToB,v6
//	ld_v	convToConst,v7

	mul_sv	bright_r_r0,v4,>>#24,v4	; apply the brightness amounts to each component
	mul_sv	bright_g_r1,v5,>>#24,v5
	mul_sv	bright_b_r2,v6,>>#24,v6

;;; Can't do this here.  For non-translucent pixels we want to apply RGB, but not translucency.
.if (0)
;	ld_s	spr_trans,trans_fg_r0
;	nop
;	and	#$FFFF,trans_fg_r0
;	lsl	#16,trans_fg_r0		; turn 18.14 into 8.24...
;	asr	#6,trans_fg_r0		; ...with sign intact
	ld_s	sprTransFG,trans_fg_r0
	nop
	
	mul_sv	trans_fg_r0,v4,>>#24,v4	; apply the translucency to the foreground matrix
	mul_sv	trans_fg_r0,v5,>>#24,v5
	mul_sv	trans_fg_r0,v6,>>#24,v6
.endif
	
	st_v	v4,convYToFG
	st_v	v5,convCrToFG
	st_v	v6,convCbToFG

	;; 
	;; Now see if we should combine YCrCb-to-RGB with RGB-to-YCrCb
	;; 
	ld_s	spr_hint,r0
	nop
	btst	#SPR_HINT_MATPREP_BOTH,r0
	bra	eq,`done,nop

	;; Mirror the matrix about the diagonal
{	copy	r17,r20
	mv_s	r20,r17
}	
{	copy	r18,r24
	mv_s	r24,r18
}	
{	copy	r19,r28
	mv_s	#0,r19
}	
{	copy	r22,r25
	mv_s	r25,r22
}	
{	copy	r23,r29
	mv_s	#0,r23
}	
{	copy	r27,r30
	mv_s	#0,r27
}
	mv_s	#1,r0
	lsl	#24,r0,r31
	lsl	#15,r0
//	mv_s	#1<<24,r31
//	mv_s	#1<<15,r0
	copy	r0,r1
	copy	r0,r2
	ld_v	convToY,v1		; get the matrix to go from RGB-to-YCrCb
	copy	r0,r3
	
	dotp	v1,v4,>>#24,r8		; combine with previous matrix so we have one that
	dotp	v1,v5,>>#24,r9		; goes YCrCb->RGB->YCrCb applying brightness along
	dotp	v1,v6,>>#24,r10		; the way.
	dotp	v1,v7,>>#24,r11
	
	ld_v	convToCr,v1		; hide nop
	add_sv	v0,v2
	st_v	v2,convYToFG

	dotp	v1,v4,>>#24,r8
	dotp	v1,v5,>>#24,r9
	dotp	v1,v6,>>#24,r10
	dotp	v1,v7,>>#24,r11
	
	ld_v	convToCb,v1		; hide nop
	add_sv	v0,v2
	st_v	v2,convCrToFG
	
	dotp	v1,v4,>>#24,r8
	dotp	v1,v5,>>#24,r9
	dotp	v1,v6,>>#24,r10
	dotp	v1,v7,>>#24,r11
	nop
	add_sv	v0,v2
	st_v	v2,convCbToFG

`done:
	pop	v7
	pop	v6
	pop	v5
	pop	v4
	pop	v3
	pop	v2
	pop	v1
	
	rts	nop

endMatPrep: