;;; ================================================================================================
;;; 
;;; Draws a translucent box that does not require saturation.  The color passed in must have
;;; already been scaled by the tranlucency factor.
;;; 
;;; ================================================================================================

	.overlay spBoxTl
	.origin pixgen
	
	.include "sp_def.i"

;	xMin_r4		= r4
;	xMax_r5		= r5
;	yMin_r6		= r6
;	yMax_r7		= r7

	boxInfo_v2	= v2
	xy1_r8		= r8
	xy2_r9		= r9
	color_r10	= r10
	transBG_r11	= r11

	pix_v3		= v3
	pix_r12		= r12

	tmpBoxInfo_v4	= v4
	
	x1_r20		= r20
	y1_r21		= r21
	x2_r22		= r22
	y2_r23		= r23
	xCur_r20	= r20
	yCur_r21	= r21
	wRemaining_r22	= r22
	hRemaining_r23	= r23
	xStart_r24	= r24
	w_r25		= r25
	wBlock_r26	= r26
	hBlock_r27	= r27
	
	buffLoad_r28	= r28
	buffDraw_r29	= r29
	buffWrite_r30	= r30
	
pixelTypeYarBox:
	push	v2
	push	v3
	push	v4
	push	v5
	push	v6
	push	v7

	;; Set up the DMA as much as we can
	ld_s	fbDMAFlags,r0			; get dma flags
	ld_s	fbAddress,r1			; get frame buffer pointer
	st_s	r0,dma__cmd+0			; store read flags
	st_s	r1,dma__cmd+4			; store read FB address
	bclr	#13,r0
	st_s	r0,dma__cmd2+0			; store write flags
	st_s	r1,dma__cmd2+4			; store write FB address

	;; Load the first box info
	ld_v	object+0,boxInfo_v2

	;; Get clipping info
	ld_s	xclip,xmin_r4			; pick up the X clip params
	ld_s	yclip,ymin_r6			; pick up the Y clip params
	lsr	#16,xmin_r4,xmax_r5		; extract max X
	lsr	#16,ymin_r6,ymax_r7		; extract max Y
	bits	#15,>>#0,xmin_r4		; and minimum X
	bits	#15,>>#0,ymin_r6		; and minimum Y

	;; Initialize triple buffer
	mv_s	#(srcBuffLoad+(  0<<2)),buffLoad_r28
	mv_s	#(srcBuffLoad+( 64<<2)),buffDraw_r29
	mv_s	#(srcBuffLoad+(128<<2)),buffWrite_r30

`nextBox:
	;; Initialize some registers
	mv_s	#0,wBlock_r26
	mv_s	#0,hBlock_r27

	;; Get signed box coordinates
	copy	xy1_r8,x1_r20
	copy	xy1_r8,y1_r21
	asl	#16,x1_r20
	asr	#16,x1_r20
	asr	#16,y1_r21
	copy	xy2_r9,x2_r22
	copy	xy2_r9,y2_r23
	asl	#16,x2_r22
	asr	#16,x2_r22
	asr	#16,y2_r23

	;; Check if the box is within the slice
	cmp	xmin_r4,x2_r22
{	bra	lt,`skipBox
	cmp	xmax_r5,x1_r20
}
{	bra	gt,`skipBox
	cmp	ymin_r6,y2_r23
}
{	bra	lt,`skipBox
	cmp	ymax_r7,y1_r21
}
	bra	gt,`skipBox

	;; Clip the box to the view
	cmp	xmin_r4,x1_r20
	bra	lt,`noclipx1,nop
	mv_s	xmin_r4,x1_r20
`noclipx1:
	cmp	xmax_r5,x2_r22
	bra	gt,`noclipx2,nop
	copy	xmax_r5,x2_r22
`noclipx2:
	cmp	ymin_r6,y1_r21
	bra	lt,`noclipy1,nop
	copy	ymin_r6,y1_r21
`noclipy1:
	cmp	ymax_r7,y2_r23
	bra	gt,`noclipy2,nop
	copy	ymax_r7,y2_r23
`noclipy2:

	;; Get the width/height to iterate over in this slice
	copy	x1_r20,xStart_r24		; save starting X coordinate
	sub	x1_r20,x2_r22,wRemaining_r22	; generate box width
	sub	y1_r21,y2_r23,hRemaining_r23	; generate box height
	copy	wRemaining_r22,w_r25		; save the overall width

`firstBlock:
	jsr	`preloadBlock,nop
	jsr	mdma_finished,nop
	jsr	`swapBuffers,nop
	
`nextBlock:
	jsr	`preloadBlock,nop
	jsr	`processBlock,nop
	jsr	mdma_finished,nop
	jsr	`swapBuffers,nop
	st_s	#dma__cmd2,mdmacptr		; write out the processed pixels
	bra	c0ne,`nextBlock,nop

`skipBox:
	;; Shift everything down
	ld_v	object+32,tmpBoxInfo_v4
	ld_v	object+16,boxInfo_v2
	st_v	tmpBoxInfo_v4,object+16
	st_s	#$FFFFFFFF,object+32+8		; store all ones in the last color spot
	cmp	#$FFFFFFFF,color_r10		; if the current color is all ones then we're done
	bra	ne,`nextBox

	jsr	mdma_finished,nop
	
	pop	v7
	pop	v6
	pop	v5
	pop	v4
	pop	v3
	pop	v2
	
	pop	v0,rz
	nop
	jmp	spriteDoneOuter,nop

	;;
	;; Swap the triple buffer (load, draw, write)
	;; 
`swapBuffers:
{	rts
	mv_s	buffLoad_r28,r0
	copy	buffWrite_r30,buffLoad_r28
}
	copy	buffDraw_r29,buffWrite_r30
	copy	r0,buffDraw_r29

	;;
	;; Start the DMA to preload the next batch of pixels
	;; 
`preloadBlock:
	;; Save the number of pixels for the previous block
	copy	wBlock_r26,r0
	mul	hBlock_r27,r0,>>#0,r0
	nop
	st_s	r0,rc0				; save number of pixels in rc0
	;; Advance the block information based on the previous block
	add	wBlock_r26,xCur_r20		; advance x coordinate
	sub	wBlock_r26,wRemaining_r22	; decrement remaining width
	bra	ne,`gotBlockInfo,nop		; if row is not done then we have our info
	copy	xStart_r24,xCur_r20		; row is done, reset the coordinate
	copy	w_r25,wRemaining_r22		; and width
	add	hBlock_r27,yCur_r21		; advance y coordinate
	sub	hBlock_r27,hRemaining_r23	; decrement remaining height
	bra	eq,`skipPreload,nop		; if height is done then do not preload next block
`gotBlockInfo:
	;; Make sure the DMA command can be modified
	jsr	mdma_wait,nop
	;; Update the write DMA command based on the previous read
	ld_s	dma__cmd+8,r0			; x/w
	ld_s	dma__cmd+12,r1			; y/h
	ld_s	dma__cmd+16,r2			; MPE address
	st_s	r0,dma__cmd2+8
	st_s	r1,dma__cmd2+12
	st_s	r2,dma__cmd2+16
	;; Create the DMA command
	copy	wRemaining_r22,wBlock_r26	; clamp block width and height to 8
	copy	hRemaining_r23,hBlock_r27
	cmp	#8,wBlock_r26
	bra	le,`noClampW,nop
	mv_s	#8,wBlock_r26
`noClampW:
	cmp	#8,hBlock_r27
	bra	le,`noClampH,nop
	mv_s	#8,hBlock_r27
`noClampH:
	;; Create the DMA command
	copy	xCur_r20,r0
	or	wBlock_r26,>>#-16,r0
	st_s	r0,dma__cmd+8			; x/w
	copy	yCur_r21,r0
	or	hBlock_r27,>>#-16,r0
	st_s	r0,dma__cmd+12			; y/h
	st_s	buffLoad_r28,dma__cmd+16	; destination address
	rts
	st_s	#dma__cmd,mdmacptr		; start dma
	st_s	#1,rc0				; non-zero = not the end
`skipPreload:
	rts
	st_s	#0,rc0				; zero = no preload performed
	nop
	
	;;
	;; Process the pixels that have been read
	;; 
`processBlock:
	copy	buffDraw_r29,r0
`nextPixel:
	ld_p	(r0),pix_v3			; read pixel YCrCb components
	dec	rc0
	mul_p	transBG_r11,pix_v3,>>#16,pix_v3	; scale the pixel by the specified amount
	nop
	st_p	pix_v3,(r0)			; store it back
	ld_s	(r0),pix_r12			; load the pixel as a scalar
	nop
	add	color_r10,pix_r12		; add in the box color
	bra	c0ne,`nextPixel
	st_s	pix_r12,(r0)			; store the final color back
	add	#4,r0				; increment pixel pointer
	rts	nop
	
pixelTypeYarBox_end: