;;; ================================================================================================
;;; 
;;; Draws gradated lines (one pixel wide).  Each sprite structure can store the components for up 
;;; to two lines.  Gradated lines are lines that have different colors on each end point and
;;; interpolated between them along the segment.
;;; 
;;; ================================================================================================

	.overlay spGLine
	.origin pixgen
	
	.include "sp_def.i"

.if (0)
	majCur_r2	= r2
	minCur_r3	= r3

	majorAxis_v1	= v1
	majMin_r4	= r4
	majMax_r5	= r5
	maj0_r6		= r6
	maj1_r7		= r7
	
	minorAxis_v2	= v2
	minMin_r8	= r8
	minMax_r9	= r9
	min0_r10	= r10
	min1_r11	= r11

	pixelCount_r12	= r12
	prevSegMaj_r13	= r13
	prevSegMin_r14	= r14
.endif
	buffPtr_r15	= r15

	lineinfo1_v4	= v4
.if (0)
	maj_coord_r16	= r16
	min_coord_r17	= r17
	dMin_dMaj_r18	= r18
.endif
	colorStart_r19	= r19
	lineinfo2_v5	= v5
	colorStep_v5	= v5
	colorStepY_r20	= r20
	colorStepCr_r21	= r21
	colorStepCb_r22	= r22
	buffPtr1_r23	= r23

	colorCur_v6	= v6
	colorCurY_r24	= r24
	colorCurCr_r25	= r25
	colorCurCb_r26	= r26
	buffPtr2_r27	= r27
	
	
pixelTypeYarGLine:
	push	v2
	push	v3
	push	v4
	push	v5
	push	v6
	push	v7

	;; Set up the DMA as much as we can
	ld_s	fbDMAFlags,r0			; get dma flags
	ld_s	fbAddress,r1			; get frame buffer pointer
	st_s	r0,dma__cmd+0
	st_s	r1,dma__cmd+4

	eor	const0_r1,const0_r1

	ld_v	object+0,lineinfo1_v4		; get the line end points, slopes, and color
	ld_v	object+16,lineinfo2_v5		; ...
	ld_p	gline_colorStart,colorCur_v6	; load the color as color components
	nop
	
	mv_s	#source_tile1,buffPtr1_r23	; double buffer the DMAs
	mv_s	#source_tile2,buffPtr2_r27
	copy	buffPtr1_r23,buffPtr_r15	; current buffer

	;; Load clipping region assuming horizontal line for now
	ld_s	xclip,majMin_r4			; pick up the X clip params
	ld_s	yclip,minMin_r8			; pick up the Y clip params
	and	#$FFFF0000,majMin_r4,majMax_r5	; get max X in 16.16 format
	and	#$FFFF0000,minMin_r8,minMax_r9	; get max Y in 16.16 format
	lsl	#16,majMin_r4			; get min X in 16.16 format
	lsl	#16,minMin_r8			; get min Y in 16.16 format

	btst	#0,dMin_dMaj_r18		; bit 1 tells if we should swap major/minor axes
	bra	eq,`NoSwapMajorMinor1,nop
	mv_v	majorAxis_v1,v7
	mv_v	minorAxis_v2,majorAxis_v1
	mv_v	v7,minorAxis_v2
`NoSwapMajorMinor1:

	;; Extract the major and minor coordinates for each end point
	lsl	#16,maj_coord_r16,maj0_r6
	lsl	#16,min_coord_r17,min0_r10
	and	#$FFFF0000,maj_coord_r16,maj1_r7
	and	#$FFFF0000,min_coord_r17,min1_r11

	;; Check if first major coordinate is greater than the maximum.
	cmp	majMax_r5,maj0_r6		; skip if starting point is beyond the clip region
	bra	gt,`skipLine
	;; Check if second major coordinate is less than the minimum.
	cmp	majMin_r4,maj1_r7		; skip if starting point is beyond the clip region
	bra	lt,`skipLine
	;; Check if both minor coordinates are less than the minimum.
	cmp	minMin_r8,min0_r10
	bra	ge,`notLessThanMin
	cmp	minMin_r8,min1_r11
	bra	lt,`skipLine
`notLessThanMin:
	;; Check if both minor coordinates are greater than the maximum
	cmp	minMax_r9,min0_r10
	bra	le,`notMoreThanMax
	cmp	minMax_r9,min1_r11
	bra	gt,`skipLine
`notMoreThanMax:
	add	#fix(0.5,16),maj1_r7		; move to middle of the pixel
	add	#fix(0.5,16),majMax_r5		; ...
	add	#fix(0.5,16),minMin_r8		; ...
	add	#fix(0.5,16),minMax_r9		; ...
	copy	maj0_r6,majCur_r2		; assume we're starting at the point X0,Y0
	copy	min0_r10,minCur_r3
	cmp	majMin_r4,maj0_r6
	bra	ge,`gotStartPoint
	sub	maj0_r6,majMin_r4,r0
{	mul	dMin_dMaj_r18,r0,>>#16,r0
	lsl	#16,r0,r1			; mul_p wants the value in top 16 bits
}
	copy	majMin_r4,majCur_r2		; grab the new starting point
	add	r1,minCur_r3			; add X offset to this point
	mul_p	r1,colorStep_v5,>>#16,v7
	nop
	add_p	v7,colorCur_v6
	
`gotStartPoint:
	add	#fix(0.5,16),majCur_r2		; start in the middle of the pixel
	add	#fix(0.5,16),minCur_r3		; ...

	;;
	;; This is the main line drawing loop.  Each line is composed of segments.  A segment is
	;; a set of pixels that share one coordinate, either X or Y.  For instance:
	;;
	;;          *****
	;;               ******
	;;                     *****
	;;
	;; The above line has three segments.  The first and third have five pixels and the
	;; middle one has six.  We traverse along, keeping a count of pixels until the shared
	;; coordinate changes (Y in this case).  Then we write out the previous segment and start
	;; a new one.
	;; 
`nextSegment:
	eor	pixelCount_r12,pixelCount_r12	; starting a new segment - zero the length
	lsr	#16,majCur_r2,prevSegMaj_r13	; save the starting points of this segment...
	lsr	#16,minCur_r3,prevSegMin_r14	; ...
	
`nextPixel:
	;; Increment the pixel count as well as the coordinates for the next point.
	add	#1,pixelCount_r12
	st_p	colorCur_v6,(buffPtr_r15)	; store the current color value
	add	#4,buffPtr_r15
	add	#fix(1,16),majCur_r2		; increment major coordinate
	add	dMin_dMaj_r18,minCur_r3		; increment minor coordinate
	add_p	colorStep_v5,colorCur_v6	; increment color
	
	;; This checks if we moved to a new minor coordinate (new segment).  If the coordinates
	;; are not equal then we are done with this segment.
	lsr	#16,minCur_r3,r0
	cmp	r0,prevSegMin_r14
	bra	ne,`writeSegment,nop

	;; Check if we hit the suggested limit of 64 scalars to transmit at one time.
	cmp	#64,pixelCount_r12
	bra	ge,`writeSegment

	;; Make sure we haven't hit the end of the line.
	cmp	maj1_r7,majCur_r2
	bra	gt,`writeSegment
	
	;; Make sure we're still within the slice.  If not, fall through and write out this segment.
	cmp	majMax_r5,majCur_r2
	bra	le,`nextPixel,nop

`writeSegment:
	;; First ensure that the segment is inside the clipping region.
	sub	dMin_dMaj_r18,minCur_r3		; step back to the previous segment for these checks
	cmp	minMin_r8,minCur_r3		; Handle the case when the segment is in the slice
{	bra	lt,`dontWrite			; ...in the major direction, but not minor.  Ideally
	cmp	minMax_r9,minCur_r3		; ...we'd just skip this segment, but to do that we
}
{	bra	gt,`dontWrite			; ...must iterate through the pixels normally.
	add	dMin_dMaj_r18,minCur_r3		; increment back to where we were
}
	nop
	nop

	;; Wait for previous DMA to be accepted.
	push	v0
	jsr	mdma_wait,nop
	pop	v0

	;; Construct the coordinates
	or	pixelCount_r12,>>#-16,prevSegMaj_r13	; merge segment length in major direction
	bset	#16,prevSegMin_r14			; one pixel wide in minor direction
	
	btst	#0,dMin_dMaj_r18		; bit 1 tells if we should swap major/minor axes
	bra	eq,`NoSwapMajorMinor2
	st_s	prevSegMaj_r13,dma__cmd+8
	st_s	prevSegMin_r14,dma__cmd+12
	st_s	prevSegMaj_r13,dma__cmd+12
	st_s	prevSegMin_r14,dma__cmd+8
`NoSwapMajorMinor2:
	st_s	buffPtr1_r23,dma__cmd+16
	st_s	#dma__cmd,mdmacptr

	copy	buffPtr2_r27,buffPtr_r15	; current buffer
	copy	buffPtr1_r23,buffPtr2_r27	; swap buffer pointers
	copy	buffPtr_r15,buffPtr1_r23
	
`dontWrite:
	;; Perform the same comparisons as was done above to see if we should fall through again.
	cmp	maj1_r7,majCur_r2		; check for end of line
	bra	gt,`doneLine
	
	cmp	majMax_r5,majCur_r2
	bra	le,`nextSegment,nop

`doneLine:
`skipLine:
	jsr	mdma_wait,nop
	
	pop	v7
	pop	v6
	pop	v5
	pop	v4
	pop	v3
	pop	v2
	
	pop	v0,rz
	nop
	jmp	spriteDoneOuter,nop

pixelTypeYarGLine_end: