;;; ================================================================================================
;;; 
;;; Draws single pixel points.  Each sprite structure can store the coordinates for up to twelve
;;; points.  They must all share the same color and not have translucency.
;;; 
;;; ================================================================================================

	.overlay spPoint
	.origin pixgen
	
	.include "sp_def.i"

	const0_r1	= r1
	xcur_r2		= r2
	ycur_r3		= r3
	xmin_r4		= r4
	xmax_r5		= r5
	ymin_r6		= r6
	ymax_r7		= r7
	
pixelTypeYarPoint:
	push	v2
	push	v3
	push	v4
;	push	v5
;	push	v6
;	push	v7

	;; Set up the DMA as much as we can
	ld_s	fbDMAFlags,r0			; get dma flags
	ld_s	fbAddress,r1			; get frame buffer pointer
	ld_s	spr_tint,r2			; get point color
	or	#(1<<27),r0			; set direct bit
	st_s	r0,dma__cmd+0
	st_s	r1,dma__cmd+4
	and	#$FFFFFF00,r2			; mask out hint bits
	st_s	r2,dma__cmd+16

	;; Now load all the potential point coordinates from the sprite structure
	ld_v	object+0,v2
	ld_v	object+16,v3
	ld_v	object+32,v4

	eor	const0_r1,const0_r1		; constant zero
	
	ld_s	xclip,xmin_r4			; pick up the X clip params
	ld_s	yclip,ymin_r6			; pick up the Y clip params
	lsr	#16,xmin_r4,xmax_r5		; extract max X
	lsr	#16,ymin_r6,ymax_r7		; extract max Y
	bits	#15,>>#0,xmin_r4		; and minimum X
	bits	#15,>>#0,ymin_r6		; and minimum Y

`nextpoint:
{	mv_s	r8,xcur_r2			; get the next point coordinates
	lsr	#16,r8,ycur_r3
}
	bits	#15,>>#0,xcur_r2

	cmp	xmin_r4,xcur_r2			; see if this point is within the slice
{	bra	lt,`skippoint
	cmp	xmax_r5,xcur_r2
}
{	bra	gt,`skippoint
	cmp	ymin_r6,ycur_r3
}
{	bra	lt,`skippoint
	cmp	ymax_r7,ycur_r3
}
	bra	gt,`skippoint

	;; Got here so point must fall within this slice.  Go ahead and draw it.
	or	#(1<<16),xcur_r2		; construct x and y parameters
	or	#(1<<16),ycur_r3
	
	;; Wait until we can modify the DMA command buffer
	push	v0
	jsr	mdma_wait,nop
	pop	v0
	nop

	st_s	xcur_r2,dma__cmd+8
	st_s	ycur_r3,dma__cmd+12
	st_s	#dma__cmd,mdmacptr		; start dma

`skippoint:
	;; Shift everything down one register
{	copy	r9,r8
	mv_s	r10,r9
	addm	r11,const0_r1,r10
}
{	copy	r12,r11
	mv_s	r13,r12
	addm	r14,const0_r1,r13
}
	cmp	#$FFFFFFFF,r8
{	copy	r15,r14
	mv_s	r16,r15
	addm	r17,const0_r1,r16
	bra	ne,`nextpoint
}
{	copy	r18,r17
	addm	r19,const0_r1,r18
	mv_s	#$FFFFFFFF,r19
}

`done:
	jsr	mdma_wait,nop
	
;	pop	v7
;	pop	v6
;	pop	v5
	pop	v4
	pop	v3
	pop	v2
	
	pop	v0,rz
	nop
	jmp	spriteDoneOuter,nop

pixelTypeYarPoint_end: