;;; ================================================================================================
;;; 
;;; Yaroze-style renderer
;;;
;;; Transparency:  YES
;;; Translucency:  YES
;;; Brightness:    YES
;;; Source format: RGB
;;;
;;; Draws translucent pixels that do not require any special saturation.  This only occurs when
;;; both fg and bg translucency values are greater than zero, their sum is less than or equal to 
;;; one, and the RGB brightness scales are less than or equal to one.
;;;
;;; This is the same as spYarTlNoSatAlpha.s with the exception that source sprite pixels are 
;;; assumed to already be in RGB rather than YCrCb.  This saves some multiplies.
;;;
;;; Num ticks:
;;;   Transparent - 9
;;;   Translucent - 14
;;;   Normal      - 12
;;;
;;; ================================================================================================

.if (STANDALONE == 0)
	.overlay spTlRA
	.origin pixgen
.endif
	
	.include "sp_def.i"

pixelTypeYarTlNoSatAlpha_RGB:

.if (PERFMON_ENABLE == 1)
	ld_s	perfmonPixels+16,r0
	nop
	add	dma_len,r0
	st_s	r0,perfmonPixels+16
.endif

	jsr	read_dest,nop

	push	v2
	push	v3
;	push	v4		; v4 is saved before getting here
	push	v5
	push	v6
	push	v7
	
{	st_s	inpBufPtr_r16,xybase		; the destination pointer that was passed in
	eor	pixelMask_r6,pixelMask_r6	; clear the mask
}
{	ld_s	sprTransFG,fgTrans_r4		; get background scale amount
	copy	tosource[0],ruInc_r0		; save these since we're destroying v7...
	addm	temp_zero_r6,tosource[2],rvInc_r1 ; ...
}
{;	ld_s	sprTransBG,bgTrans_r7		; get background scale amount
	ld_s	sprTransFGPix,fgTransPix_r7	; foreground scale for pixel op
	copy	inp1dot24_r19,r11		; constant that was passed in
	addm	temp_zero_r6,inp1dot24_r19,r15	; ...
}
{	ld_v	convYToBG,convToR_v5		; matrix to convert background to RGB
	bra	`next_pixel
}
	ld_v	convCrToBG,convToG_v6		; ...
	ld_v	convCbToBG,convToB_v7		; ...
	
`transparent_pixel:
{	st_s	bgPixel_r3,(xy)			; store the background pixel
	addr	#(1<<16),rx			; advance to next dest pixel
}
	ld_v	convYToBG,convToR_v5		; matrix to convert the pixel to RGB...
	ld_v	convCrToBG,convToG_v6		; ...while applying brightness scales
;	ld_v	convCbToBG,convToB_v7		; ...
	
`next_pixel:
{	ld_s	(uv),srcPixel_r2		; get the pixel as a scalar
	bra	c0eq,`done,nop
}
`next_pixel2:
	ld_p	(bgPtr_r5),bgPixel_v3		; get the pixel components separately
{	cmp	#$10808001,srcPixel_r2		; check for transparency
	ld_p	(uv),fgPixelRGB_v4		; ...and components separately
}
{	bra	eq,`transparent_pixel		; branch if transparent
	dec	rc0				; decrement number of pixels left
	rot	#1,pixelMask_r6			; make room for next mask bit
	ld_s	(bgPtr_r5),bgPixel_r3		; ...and as a scalar
	dotp	bgPixel_v3,convYToBG_v5,>>#24,bgPixelR_r8
}
{	btst	#0,srcPixel_r2
	addr	ruInc_r0,ru			; advance to next sprite pixel...
	dotp	bgPixel_v3,convCrToBG_v6,>>#24,bgPixelG_r9
	ld_v	convToY,convToY_v5		; get matrix to convert from RGB to YCrCb
}
{	bra	eq,`notTranslucent		; skip the translucency multiplies if not trans
	add	#4,bgPtr_r5			; advance to next background pixel
	addr	rvInc_r1,rv			; ...
	dotp	bgPixel_v3,convCbToBG_v7,>>#24,bgPixelB_r10
	ld_v	convToCr,convToCr_v6
}
{	ld_v	convToCb,convToCb_v7
	mul_p	fgTrans_r4,fgPixelRGB_v4,>>#24,fgPixelRGB_v4	; apply translucency to fg
	eor	bgPixelA_r15,bgPixelA_r15
}
	bclr	#8,bgPixel_r3
	lsl	#23,bgPixel_r3,bgPixelA_r15	; not opaque - put alpha in [30:23]
{	mul	fgTransPix_r7,bgPixelA_r15,>>#24,bgPixelA_r15
	add_p	bgPixelRGB_v2,fgPixelRGB_v4,destPixelRGB_v4	; combine fg & bg
}

`notTranslucent:
	;; Convert final pixel back to YCrCb
{	dotp	destPixelRGB_v4,convToY_v5,>>#24,destPixelY_r12
	ld_v	convYToBG,convYToBG_v5		; matrix to convert the pixel to RGB...
	bset	#0,pixelMask_r6			; set mask bit
}
{	dotp	destPixelRGB_v4,convToCr_v6,>>#24,destPixelCr_r13
	ld_v	convCrToBG,convCrToBG_v6	; ...while applying both tint and...
	rot	#1,pixelMask_r6			; rotate the mask to make room for the next pixel
}
{	dotp	destPixelRGB_v4,convToCb_v7,>>#24,destPixelCb_r14
	ld_v	convCbToBG,convCbToBG_v7	; ...translucency
	rot	#-1,pixelMask_r6		; undo the rotation since we went too far
	bra	c0ne,`next_pixel2		; jump back to do the next pixel if not done
}
{	ld_s	(uv),srcPixel_r2
	lsl	#1,bgPixelA_r15			; put alpha in [31:24]
}
{	st_pz	destPixel_v3,(xy)
	addr	#(1<<16),rx			; advance to next dest pixel
	copy	inp1dot24_r19,r15		; restore this since we overwrote it with alpha
}
	
`done:
	pop	v7
	pop	v6
	pop	v5
;	pop	v4		; v4 is restored after returning
	pop	v3
	bra	out
	pop	v2
	nop
	
endPixelTypeYarTlNoSatAlpha_RGB:
