;;; ================================================================================================
;;; 
;;; Yaroze-style renderer
;;;
;;; Transparency: YES
;;; Translucency: YES
;;; Brightness:   YES
;;;
;;; Draws translucent pixels that require saturation only after the final blend.  This occurs when
;;; both fg and bg translucency values are greater than zero, and their sum when combined with RGB
;;; brightness scales is less than or equal to two.
;;;
;;; This is the same as spYarTlSatEnd.s with the exception that source sprite pixels are 
;;; assumed to already be in RGB rather than YCrCb.  This saves some multiplies.
;;;
;;; Num ticks:
;;;   Transparent - 6
;;;   Translucent - 22
;;;   Normal      - 15
;;;
;;; ================================================================================================

	.overlay spTlER
	.origin pixgen
	
	.include "sp_def.i"
	
pixelTypeYarTlSatEnd_RGB:

.if (PERFMON_ENABLE == 1)
	ld_s	perfmonPixels+20,r0
	nop
	add	dma_len,r0
	st_s	r0,perfmonPixels+20
.endif

	jsr	read_dest,nop

	push	v2
	push	v3
;	push	v4		; v4 is saved before getting here
	push	v5
	push	v6
	push	v7
	
	st_s	inpBufPtr_r16,xybase
	
	eor	pixelMask_r6,pixelMask_r6	; clear the mask
	
	ld_s	sprTransFG,fgTrans_r4		; get background scale amount
	ld_s	sprTransBG,bgTrans_r7		; get background scale amount
;	ld_s	sprTransFGPix,fgTransPix_r7	; foreground scale for pixel op
	
	copy	tosource[0],ruInc_r0		; save these since we're destroying v7...
	copy	tosource[2],rvInc_r1		; ...

	copy	inp1dot24_r19,r11		; constant that was passed in
	copy	inp1dot24_r19,r15		; ...
	
	ld_v	convToR,convToR_v5		; matrix to convert the pixel to RGB...
	ld_v	convToG,convToG_v6		; ...while applying brightness scales
	ld_v	convToB,convToB_v7		; ...
	
`next_pixel:
{	ld_s	(uv),srcPixel_r2		; get the pixel components as a scalar...
	bra	c0eq,`done,nop
}
{	ld_s	(bgPtr_r5),bgPixel_r3		; get the pixel components as a scalar...
;	add	#4,bgPtr_r5			; advance to next background pixel
}
	cmp	#$10808001,srcPixel_r2		; check for transparency
{	bra	eq,`next_pixel
	st_s	bgPixel_r3,(xy)			; speculatively store the background pixel
	addr	#(1<<16),rx			; advance to next dest pixel
	dec	rc0				; decrement number of pixels left
	rot	#1,pixelMask_r6			; make room for next mask bit
}
	
{	ld_p	(uv),fgPixelRGB_v4		; ...and separately
;	sub	#4,bgPtr_r5			; advance to next background pixel
	addr	ruInc_r0,ru			; advance to next sprite pixel...
}
{	ld_p	(bgPtr_r5),bgPixel_v3		; ...and separately
	add	#4,bgPtr_r5			; advance to next background pixel
	addr	rvInc_r1,rv			; ...
}

	btst	#0,srcPixel_r2
{	bra	eq,`notTranslucent		; skip the translucency multiplies if not trans
	dotp	bgPixel_v3,convToR_v5,>>#24,bgPixelR_r8
	ld_v	convToY,convToY_v5		; get matrix to convert from RGB to YCrCb
}
{	dotp	bgPixel_v3,convToG_v6,>>#24,bgPixelG_r9
	ld_v	convToCr,convToCr_v6
}
{	dotp	bgPixel_v3,convToB_v7,>>#24,bgPixelB_r10
	ld_v	convToCb,convToCb_v7
	eor	bgPixelA_r15,bgPixelA_r15			; assume opaque for now
}
	mul_p	fgTrans_r4,fgPixelRGB_v4,>>#24,fgPixelRGB_v4	; apply translucency to fg
	
	;; The next line was commented out for some reason, but seems to me it should be uncommented
	mul_p	bgTrans_r7,bgPixelRGB_v2,>>#24,bgPixelRGB_v2	; apply translucency to bg
	bset	#24,bgPixelA_r15				; not opaque - set STP bit
	add_p	bgPixelRGB_v2,fgPixelRGB_v4,destPixelRGB_v4	; combine fg & bg
	sat	#31,destPixelR_r16
	sat	#31,destPixelG_r17
	sat	#31,destPixelB_r18

`notTranslucent:
	;; Convert final pixel back to YCrCb
{	dotp	destPixelRGB_v4,convToY_v5,>>#24,destPixelY_r12
	ld_v	convToR,convToR_v5		; matrix to convert the pixel to RGB...
}
{	dotp	destPixelRGB_v4,convToCr_v6,>>#24,destPixelCr_r13
	ld_v	convToG,convToG_v6		; ...while applying both tint and...
	rot	#-1,pixelMask_r6		; undo the rotation since we went too far
}
{	dotp	destPixelRGB_v4,convToCb_v7,>>#24,destPixelCb_r14
	ld_v	convToB,convToB_v7		; ...translucency
	bra	c0ne,`next_pixel		; jump back to do the next pixel if not done
	bset	#0,pixelMask_r6			; set mask bit
}
{	addr	#(-1<<16),rx			; advance to next dest pixel
	rot	#1,pixelMask_r6			; rotate the mask to make room for the next pixel
}
{	st_pz	destPixel_v3,(xy)
	addr	#(1<<16),rx			; advance to next dest pixel
	copy	inp1dot24_r19,r15		; restore this since we overwrote it with alpha
}
	
`done:
	pop	v7
	pop	v6
	pop	v5
;	pop	v4		; v4 is restored after returning
	pop	v3
	bra	out
	pop	v2
	nop
	
endPixelTypeYarTlSatEnd_RGB:
	