;;; ================================================================================================
;;; 
;;; Yaroze-style renderer
;;;
;;; Transparency: YES
;;; Translucency: YES
;;; Brightness:   YES
;;;
;;; Draws translucent pixels that require signed saturation at the end.  This occurs when one of
;;; the translucency scales is negative.
;;;
;;; This differs from spYarTlSatEndZero in that we are assured that no pixels will end up greater
;;; than 1.0 for RGB.
;;;
;;; Num ticks:
;;;   Transparent - 6
;;;   Translucent - 29
;;;   Normal      - 17
;;;
;;; ================================================================================================

	.overlay spTlZ
	.origin pixgen
	
	.include "sp_def.i"
	
pixelTypeYarTlSatZero:

.if (PERFMON_ENABLE == 1)
	ld_s	perfmonPixels+28,r0
	nop
	add	dma_len,r0
	st_s	r0,perfmonPixels+28
.endif

	jsr	read_dest,nop

	push	v2
	push	v3
;	push	v4		; v4 is saved before getting here
	push	v5
	push	v6
	push	v7
	
	st_s	inpBufPtr_r16,xybase		; the destination pointer that was passed in
	
	eor	pixelMask_r6,pixelMask_r6	; clear the mask
	
	ld_s	sprTransFG,fgTrans_r4		; get background scale amount
	ld_s	sprTransBG,bgTrans_r7		; get background scale amount
;	ld_s	sprTransFGPix,fgTransPix_r7	; foreground scale for pixel op
	
	copy	tosource[0],ruInc_r0		; save these since we're destroying v7...
	copy	tosource[2],rvInc_r1		; ...

	copy	inp1dot24_r19,r11
	copy	inp1dot24_r19,r15
	
	ld_v	convYToFG,convYToFG_v5		; matrix to convert the pixel to RGB...
	ld_v	convCrToFG,convCrToFG_v6	; ...while applying brightness scales
	ld_v	convCbToFG,convCbToFG_v7	; ...
	
`next_pixel:
{	ld_s	(uv),srcPixel_r2		; ...and as a scalar
	bra	c0eq,`done,nop
}
{	ld_s	(bgPtr_r5),bgPixel_r3		; ...and as a scalar
	add	#4,bgPtr_r5			; advance to next background pixel
}
	cmp	#$10808001,srcPixel_r2		; check for transparency
{	bra	eq,`next_pixel
	st_s	bgPixel_r3,(xy)			; speculatively store the background pixel
	addr	#(1<<16),rx			; advance to next dest pixel
	dec	rc0				; decrement number of pixels left
}
	
{	ld_p	(uv),srcPixel_v2		; get the pixel components separately...
	sub	#4,bgPtr_r5			; advance to next background pixel
	addr	ruInc_r0,ru			; advance to next sprite pixel...
}
{	ld_p	(bgPtr_r5),bgPixel_v3		; get the pixel components separately...
	add	#4,bgPtr_r5			; advance to next background pixel
	addr	rvInc_r1,rv			; ...
}

{	dotp	srcPixel_v2,convYToFG_v5,>>#24,fgPixelR_r16
	ld_v	convToR,convToR_v5		; matrix to convert background to RGB
}
{	dotp	srcPixel_v2,convCrToFG_v6,>>#24,fgPixelG_r17
	ld_v	convToG,convToG_v6		; ...
}
{	dotp	srcPixel_v2,convCbToFG_v7,>>#24,fgPixelB_r18
	ld_v	convToB,convToB_v7		; ...
	btst	#0,srcPixel_r2
}

{	bra	eq,`notTranslucent		; skip the translucency multiplies if not trans
	dotp	bgPixel_v3,convToR_v5,>>#24,bgPixelR_r8
	ld_v	convToY,convToY_v5		; get matrix to convert from RGB to YCrCb
}
{	dotp	bgPixel_v3,convToG_v6,>>#24,bgPixelG_r9
	ld_v	convToCr,convToCr_v6
	bclr	#8,bgPixel_r3
}
{	dotp	bgPixel_v3,convToB_v7,>>#24,bgPixelB_r10
	ld_v	convToCb,convToCb_v7
	eor	bgPixelA_r15,bgPixelA_r15			; assume opaque for now
}
	mul_p	fgTrans_r4,fgPixelRGB_v4,>>#24,fgPixelRGB_v4	; apply translucency to fg
	
	;; The next line was commented out for some reason, but seems to me it should be uncommented
	mul_p	bgTrans_r7,bgPixelRGB_v2,>>#24,bgPixelRGB_v2	; apply translucency to bg
	bset	#24,bgPixelA_r15				; not opaque - set STP bit
{	add_p	bgPixelRGB_v2,fgPixelRGB_v4,destPixelRGB_v4	; combine fg & bg
	subm	tmp1_r8,tmp1_r8,tmp1_r8				; constant zero
}

	;;=================================================
	;; Perform signed saturation to a range of [0,max]
	subm	destPixelR_r16,tmp1_r8,tmp2_r9	; negate color component
{	sat	#1,tmp2_r9			; tmp has mask, all 0=neg, all 1=pos
	subm	destPixelG_r17,tmp1_r8,tmp3_r10
}
{	sat	#1,tmp3_r10
	subm	destPixelB_r18,tmp1_r8,tmp1_r8
}
	sat	#1,tmp1_r8
	and	tmp2_r9,destPixelR_r16		; apply mask to sat to 0 minimum
	and	tmp3_r10,destPixelG_r17
	and	tmp1_r8,destPixelB_r18
	;; Done saturation
	;;=================================================
	
`notTranslucent:
	;; Convert final pixel back to YCrCb
{	dotp	destPixelRGB_v4,convToY_v5,>>#24,destPixelY_r12
	ld_v	convYToFG,convYToFG_v5		; matrix to convert the pixel to RGB...
}
{	dotp	destPixelRGB_v4,convToCr_v6,>>#24,destPixelCr_r13
	ld_v	convCrToFG,convCrToFG_v6	; ...while applying both tint and...
}
{	dotp	destPixelRGB_v4,convToCb_v7,>>#24,destPixelCb_r14
	ld_v	convCbToFG,convCbToFG_v7	; ...translucency
	bra	c0ne,`next_pixel		; jump back to do the next pixel if not done
}
{	bset	#0,pixelMask_r6			; set mask bit
	addr	#(-1<<16),rx			; advance to next dest pixel
}
{	st_pz	destPixel_v3,(xy)
	addr	#(1<<16),rx			; advance to next dest pixel
	copy	inp1dot24_r19,r15
}
	
`done:
	pop	v7
	pop	v6
	pop	v5
//	pop	v4		; v4 is restored after returning
	pop	v3
	bra	out
	pop	v2
	nop
	
endPixelTypeYarTlSatZero:
	