;;; ================================================================================================
;;; 
;;; Yaroze-style renderer
;;;
;;; Transparency: YES
;;; Translucency: NO
;;; Brightness:   NO
;;;
;;; Transparent pixels are black pixels without the STP (semi-transparent) bit set.  Since this is
;;; the opposite sense of every other STP bit I reverse it before getting here.  So I check for
;;; 0x10808001 which is black with the bit set.
;;;
;;; Num ticks:
;;;   Normal      - 4
;;;   Transparent - 5
;;;
;;; Bug found 2004-07-31.
;;; When a sprite is extremely minimized and only draws one pixel, the renderer will still preload 
;;; the second pixel.  This may cause a data address exception if RU and RV are way out of whack 
;;; due to the sprite's extreme scale.
;;; ================================================================================================

.if (STANDALONE == 0)
	.overlay spTp
	.origin pixgen
.endif

	.include "sp_def.i"
	
pixelTypeYarTp:
	
.if (PERFMON_ENABLE == 1)
	ld_s	perfmonPixels+4,r0
	nop
	add	dma_len,r0
	st_s	r0,perfmonPixels+4
.endif

	ld_s	uvbase,r6
	nop
	st_s	r6,TempData

	st_s	inpBufPtr_r16,xybase		; set up XY register to point at dest buffer
	st_s	#(4<<20),xyctl			; do this since we didn't call matprep
{	addr	#(-1<<16),rx			; pre-decrement
	cmp	dma_len,#1
}
	
{	eor	pixelMask_r6,pixelMask_r6	; clear the mask
	bra	ne,`next_pixel
}

{	ld_s	(uv),srcPixel_r2		; load first pixel
	addr	tosource[2],rv			; advance to next source pixel...
}
	addr	tosource[0],ru			; ...continuing to advance to next source pixel
	st_s	#0,ru				; only execute these if drawing one pixel...
	st_s	#0,rv				; ...

`next_pixel:
{	cmp	#$10808001,srcPixel_r2		; do compare in advance
	bra	c0eq,outTrans			; jump out if no more pixels
	ld_s	(uv),srcPixelPre_r0		; preload next source pixel
	addr	tosource[0],ru			; ...continue advancing to next source pixel
}
{	bra	ne,`next_pixel			; not transparent, do next pixel
	dec	rc0				; decrement pixel count
	addr	tosource[2],rv			; advance to next source pixel...
	rot	#1,pixelMask_r6			; make room for new mask bit
}
{	bset	#0,pixelMask_r6
	bra	c0ne,`next_pixel		; process next pixel (transparent)
	addr	#(1<<16),rx			; advance destination pointer
}
{	st_s	srcPixel_r2,(xy)		; store the pixel speculatively
	copy	srcPixelPre_r0,srcPixel_r2	; make preloaded source pixel official
	bra	outTrans
}
	bclr	#0,pixelMask_r6
	rot	#1,pixelMask_r6			; final rotate - only gets here when fall through

endPixelTypeYarTp:
