;;; ================================================================================================
;;; 
;;; Yaroze-style renderer
;;;
;;; Transparency: YES
;;; Translucency: NO
;;; Brightness:   YES (restricted to 1.0 or less)
;;;
;;; This renderer adds brightness control to the standard transparency.  This version has the 
;;; restriction that each brightness scale must be 1.0 or less.  This guarantees no overflow and
;;; avoids having to clamp each component.
;;;
;;; Num ticks:
;;;   Normal      - 5
;;;   Transparent - 4
;;;
;;; Bug found 2004-07-31.
;;; When a sprite is extremely minimized and only draws one pixel, the renderer will still preload 
;;; the second pixel.  This may cause a data address exception if RU and RV are way out of whack 
;;; due to the sprite's extreme scale.
;;; ================================================================================================

	.overlay spTpBS
	.origin pixgen
	
.ifndef PERFMON_ENABLE
	PERFMON_ENABLE	= 0
.endif

	.include "sp_def.i"

pixelTypeYarTpBrightSimple:

.if (PERFMON_ENABLE == 1)
	ld_s	perfmonPixels+8,r0
	nop
	add	dma_len,r0
	st_s	r0,perfmonPixels+8
.endif

	push	v3
;	push	v4		; v4 is saved before getting here
	push	v5
	push	v6
	push	v7
	
	st_s	inpBufPtr_r16,xybase
	st_s	#(4<<20),xyctl
	addr	#(-1<<16),rx
	
	copy	inp1dot24_r19,r27		; fill out 4th component
	
	eor	pixelMask_r6,pixelMask_r6	; clear the mask
	
	ld_v	convYToFG,fromYToY_v3
	ld_v	convCrToFG,fromCrToCr_v4
	ld_v	convCbToFG,fromCbToCb_v5
	ld_p	(uv),srcPixel_v6		; read the pixel as YCrCb...
{	ld_s	(uv),srcPixel_r2		; ...and as a scalar
	cmp	dma_len,#1
}
{	copy	tosource[0],ruInc_r0		; save these since we're destroying v7...
	bra	ne,`next_pixel
}
{	copy	tosource[2],rvInc_r1		; ...
	addr	ruInc_r0,ru			; advance to next source pixel...
}
	addr	rvInc_r1,rv			; ...continuing to advance to next source pixel
	st_s	#0,ru				; only execute these if drawing one pixel...
	st_s	#0,rv				; ...
`next_pixel:
{	dotp	srcPixel_v6,fromYToY_v3,>>#24,dstPixelY_r28	; apply tint to Y
	cmp	#$10808001,srcPixel_r2		; check if this pixel is transparent
	bra	c0eq,`done			; jump out if done with all pixels
}
{	dotp	srcPixel_v6,fromCrToCr_v4,>>#24,dstPixelCr_r29	; apply tint to CR
	ld_s	(uv),srcPixel_r2		; get next pixel as a scalar
	bra	eq,`next_pixel			; jump back if transparent
	addr	#(1<<16),rx
	dec	rc0				; one pixel down
	rot	#1,pixelMask_r6
}
{	dotp	srcPixel_v6,fromCbToCb_v5,>>#24,dstPixelCb_r30	; apply tint to CB
	ld_p	(uv),srcPixel_v6		; get next pixel as components
	bra	c0ne,`next_pixel		; jump back if not done
	addr	ruInc_r0,ru			; advance to next pixel...
}
	addr	rvInc_r1,rv			; ...continue advancing
{	st_p	dstPixel_v7,(xy)		; overwrite with actual pixel if not transparent
	bset	#0,pixelMask_r6
}
	rot	#1,pixelMask_r6
	
`done:
	pop	v7
	pop	v6
	pop	v5
	bra	outTrans
;	pop	v4		; v4 is restored after returning
	pop	v3
	nop
	
endPixelTypeYarTpBrightSimple:
