;; =================================================================================================

	STANDALONE	= 0
	YAROZE		= 1
	PERFMON_ENABLE	= 0
	WAITCNT_ENABLE	= 0
	
SCRNWIDTH = 360
DMA_READ_BIT    = (1<<13)
DMA_PIXEL_WRITE = (3<<14)
DMA_PIXEL_READ  = (DMA_PIXEL_WRITE|DMA_READ_BIT)
DMAFLAGS   = (DMA_PIXEL_WRITE | ((SCRNWIDTH/8)<<16) | (4<<4) | (1<<11))
DMAFLAGS16 = (DMA_PIXEL_READ  | ((16/8)       <<16) | (4<<4) | (1<<11))

	kOneShotBit	= 0
	kSlicedBit	= 1
	kClearBit	= 2
	kHangBit	= 3

	;; Sprite flags
	SPR_HINT_MATPREP_ONE	= 1
	SPR_HINT_MATPREP_BOTH	= 2
	SPR_HINT_DEBUG		= 4
	SPR_HINT_RENDER_IMM	= 5

	.include "sp_def.i"
	
	.include "sprdata.s"
	
		.segment spritec
		.origin 0x20300000

	_ymin_r14 = r14
	_ymax_r15 = r15
	_slices_r16 = r16
	_zonelo_r17 = r17
	_zonehi_r18 = r18

	;; ==========================================================================================
	;;
	;; SECTION - Startup
	;;
	;; Upstream registers to save:   none
	;; Downstream registers to save: none
	;;
	;; Determine on what MPE we are running.
	;; This is done to get the jump pointer for the Comm Bus Receive routine.
	;; 
	;; ==========================================================================================
	ld_s	configa,r1
	ld_s	intvec1,r0			; preload the interrupt vector
	bits	#7,>>#8,r1
	bra	ne,`notMPE0,nop
`MPE0:
 	sub	#8,r0				; Comm Bus Receive pointer is intvec - 8
	bra	`gotCommRecvPointer,nop
`notMPE0:	
	st_s	#0x20101000,sp			; initialize stack pointer on MPE1 & 2
	mv_s	#CommRecv,r0
`gotCommRecvPointer:
	st_s	r0,CommRecvPoi

	;; ==========================================================================================
	;;
	;; SECTION - Get draw instructions (raster info, sprite list, flags, slice info)
	;; 
	;; Upstream registers to save:   none
	;; Downstream registers to save: none
	;;
	;; Main loop for initiating the sprite engine.
	;; 
	;; ==========================================================================================
main:
.if (STANDALONE == 1)
	jmp	SkipCommBus,nop
.endif
	ld_s	CommRecvPoi,r0
	nop
	jsr	(r0),nop

	;; Received the comm bus data - save it for later
	mv_v	v0,v4
	st_s	r17,qinf
	st_s	r18,mpeinfo

.if ((WAITCNT_ENABLE == 1) || (PERFMON_ENABLE == 1))
{	mv_s	#0,r0
	eor	r1,r1
}
{	mv_s	#0,r2
	eor	r3,r3
}
.endif

.if (WAITCNT_ENABLE == 1)
	st_v	v0,dmawaitcnt+0
	st_v	v0,dmawaitcnt+16
	st_v	v0,dmawaitcnt+32
.endif

.if (PERFMON_ENABLE == 1)
	st_v	v0,perfmonSprites	; covers: Sprites TilesTotal TilesDrawn & Scanlines
	st_v	v0,perfmonPixels+0	; sp_pixg0-3
	st_v	v0,perfmonPixels+16	; sp_pixg4-7
.endif

	;; ==========================================================================================
	;;
	;; SECTION - Get raster information (clip info, clear parameters)
	;; 
	;; Upstream registers to save:   none
	;; Downstream registers to save: none
	;;
	;; ==========================================================================================
	
	;; Read the raster information
	jsr	dma_read
{	mv_s	#8,r0			; raster info size
	copy	r16,r1			; r16 = commpacket[0]
}
	mv_s	#dest_screen,r2
	jsr	odma_finished,nop
SkipCommBus:

	;; ==========================================================================================
	;;
	;; SECTION - Clear screen
	;; 
	;; Upstream registers to save:   none
	;; Downstream registers to save: none
	;;
	;; ==========================================================================================
	
	;; Determine if we need to clear the screen first
	ld_s	mpeinfo,r0
	nop
	btst	#kClearBit,r0
	bra	ne,ClearScreen,nop
DoneClearScreen:

	;; ==========================================================================================
	;;
	;; SECTION - Get sprite info
	;; 
	;; Upstream registers to save:   none
	;; Downstream registers to save: none
	;;
	;; Reads sprite information.
	;; 
	;; ==========================================================================================
	
	ld_s	qinf,r17			; reload this in case it was squashed by the clear routine
	nop
nextObj:
.if (STANDALONE == 1)
	jmp	SkipObjRead,nop
.endif
	or	r17,r17				; don't render if there's no objects on the list
	bra	eq,AllDone,nop
	
	jsr	dma_read
{	mv_s	#16,r0
	copy	r17,r1				; r17 = commpacket[1]
}
{	mv_s	#qinf,r2
	add	#4,r1				; Skip past "prev"
}

{	jsr	odma_finished,nop		; must have r0, r1, & r2 free
	mv_s	#0,r1
}
	
SkipObjRead:
	ld_s	mpeinfo,r1
	nop
`hang:
	btst	#kHangBit,r1			; used for real-time debugging
	bra	ne,`hang,nop
	
	;; Get the pixel jump location and start the DMA to read the renderer
.if (STANDALONE == 0)
{	jsr	readPixgen,nop
	ld_s	rendererInfo,r0
}
.endif
//readPixgenDone:
	st_s	#pixgen,pixjump_loc
	
	ld_s	yclip,_ymin_r14			;pickup the Y clip params
	nop
	lsr	#16,_ymin_r14,_ymax_r15			;extract max-Y
	bits	#15,>>#0,_ymin_r14			;and minimum

; do multiple MPE code
	
	mv_s	#0,_slices_r16			;slice counter

slic:
	ld_s	mpeinfo,r1			; get slice size
	ld_s	mpeinfo,r2			; get our logical mpe#
	lsr	#20,r1				; slice size
	copy	r2,r3
	bits	#7,>>#8,r3			; total # of mpes
	bits	#3,>>#16,r2			; logical mpe#
	
	mul	r1,r2,>>#0,r2			; this is the base offset
	mul	r1,r3,>>#0,r3			; this is the chunk size
	nop

; start line of render zone is (chunk offset*slice number)+base offset

	mul	_slices_r16,r3,>>#0,r3
	nop
	add	r3,r2,_zonelo_r17
	add	_zonelo_r17,r1,_zonehi_r18	; get lo and hi edges of current zone
	sub	#1,_zonehi_r18			; clip is inclusive

	cmp	_zonelo_r17,_ymax_r15
	bra	lt,restore_yclip		; start>hibound; the object's done

	cmp	_zonehi_r18,_ymin_r14
	bra	gt,next_zone			; end<lobound; do next zone

; preserve the original y clip info before I mod it

	push	v3				; save original Y-clip info

	cmp	_zonelo_r17,_ymin_r14
	bra	gt,nboto,nop			; if Ymin > Zonelo, then use Ymin...	
	copy	_zonelo_r17,_ymin_r14		; else, just use zonelo.
nboto:

	cmp	_zonehi_r18,_ymax_r15
	bra	lt,ntopo,nop			; if Ymax < Zonehi then use Ymax
	copy	_zonehi_r18,_ymax_r15		; or else use zonehi.
ntopo:

; now, set _ymin and _ymax to be the Y clip params

	lsl	#16,_ymax_r15
	or	_ymax_r15,_ymin_r14
	st_s	_ymin_r14,yclip			; set them up

	push	v4
	bra	draw_sprite,nop
spriteDoneOuter:
	;; Save the renderer & source image for next time
;	ld_s	rendererInfo,r16
;	ld_s	spr_addr,r17
;	st_s	r16,prevRenderer
;	st_s	r17,prevSprAddr
	pop	v4
next_zone:

	bra	slic				; go to next slice
	pop	v3				; get back y-clip stuff too
	add	#1,_slices_r16			; enumerate next slice	

restore_yclip:
{	jsr	mdma_wait,nop
	DMAWAITCNT_ITEM 10
}

; restore yclip modified by subdivision
	lsl	#16,_ymax_r15
	or	_ymax_r15,_ymin_r14
	st_s	_ymin_r14,yclip			; set them up

	ld_s	mpeinfo,r18
	ld_s	qinf,r17
.if (STANDALONE == 0)
	btst	#kOneShotBit,r18
	bra	ne,`onshot,nop
	or	r17,r17
	bra	ne,nextObj,nop
`onshot:

AllDone:
	jsr	mdma_finished,nop
	sub_sv	v0,v0
	mv_s	#0xbaaababe,r0
	jsr	CommSend
	mv_s	#3,r4
	mv_s	#0,r5

	bra	main,nop
.else
	halt
	nop
	nop
.endif

CommRecv:
	ld_s	commctl,r4
	nop
	btst	#31,r4
	bra	eq,CommRecv,nop
	ld_s	comminfo,r5
	ld_v	commrecv,v0
	rts
	bits	#7,>>#16,r4
	bits	#7,>>#16,r5

CommSend:
	ld_s	commctl,r6		; poll for transmit data empty
	nop
	btst	#15,r6			; ready to send?
	bra	ne,CommSend,nop		; no -- keep waiting

`resend:
{	st_s	r5,comminfo
	bits	#8,>>#0,r4		; isolate just the address
}
	st_s	r4,commctl
	st_v	v0,commxmit		; write the transmit data register to start the transfer

`waitxmit:
; now wait for the data to go
	ld_io	commctl,r6
	nop
	btst	#14,r6			; check for transmit failure
{	bra	ne,`resend,nop		; failed -- retry
	btst	#5,r6			; check for transmit failure during interrupt
}
{	bra	ne,`resend,nop		; failed -- retry
	btst	#15,r6			; check for transmit buffer empty
}
	rts	eq,nop			; if it's empty, return
	bra	`waitxmit,nop

	;; =================
	hRemaining_r4	= r4
	finalY_r5	= r5
	wRemaining_r6	= r6
	clipH_r7	= r7
	xCur_r8		= r8
	yCur_r9		= r9
	wCur_r10	= r10
	hCur_r11	= r11
	tmp_r12		= r12
	sliceSize_r13	= r13
	startY_r14	= r14
	yStep_r15	= r15
	;; =================
ClearScreen:
.if (0)
	mv_s	#1,r0
`hang:
	cmp	#1,r0
	jmp	eq,`hang,nop
.endif
	
{	jsr	readPixgen,nop
	ld_s	clearRenderer,r0
}
	
	jsr	mdma_wait,nop			; wait until we can change the dma command
	
	ld_s	mpeinfo,sliceSize_r13
	ld_s	mpeinfo,r15			; get our logical mpe#
	lsr	#20,sliceSize_r13		; get slice size
	copy	r15,r14
	bits	#7,>>#8,yStep_r15		; yStep for this MPE
	bits	#3,>>#16,startY_r14		; yStart for this MPE

	mul	sliceSize_r13,startY_r14,>>#0,startY_r14	;this is the base offset
	mul	sliceSize_r13,yStep_r15,>>#0,yStep_r15		;this is the chunk size
	nop

	ld_s	fbDMAFlags,tmp_r12		; Get DMA flags for the frame buffer
	nop
	st_s	tmp_r12,dma__cmd+0

	ld_s	fbAddress,tmp_r12		; Address of frame buffer
	nop
	st_s	tmp_r12,dma__cmd+4

	ld_s	yclip,finalY_r5
	nop
	lsr	#16,finalY_r5			; Get Y coord of end of clipping region

	ld_s	yclip,yCur_r9
	nop
	bits	#15,>>#0,yCur_r9		; Initialize current Y coord
	add	startY_r14,yCur_r9		; Add offset for this MPE

	;;
	;; WARNING!
	;; This code assumes that the slice height is a multiple of the screen height.
	;; Should fix this at some point.
	;; 
	mv_s	sliceSize_r13,hRemaining_r4	; Initialize number of lines to transmit

	jsr	mdma_finished,nop		; make sure the renderer is done loading

	jmp	firstSlice,nop

nextSlice:
	add	yStep_r15,yCur_r9		; Go to end of next slice
	sub	sliceSize_r13,yCur_r9		; Back up to beginning of next slice
	
	add	yStep_r15,startY_r14		; Get next starting Y coord
	cmp	finalY_r5,startY_r14		; Are we done?
	jmp	gt,clearDone,nop

	sub	startY_r14,finalY_r5,tmp_r12	; Check if this isn't a full-height slice
	add	#1,tmp_r12
	cmp	hRemaining_r4,tmp_r12
	bra	ge,gotSlice
	copy	sliceSize_r13,hRemaining_r4	; Initialize number of lines to transmit
	nop
	copy	tmp_r12,hRemaining_r4
gotSlice:

firstSlice:
	jmp	pixgen,nop
//	.include "clrColor.s"
	
clearDone:
{	jsr	mdma_finished,nop
	DMAWAITCNT_ITEM 9
}
	bra	DoneClearScreen,nop

	.include "clrColor.s"
	.include "clrCopyFade.s"
	.include "clrCopyTile.s"
	.include "clrStars.s"
	
	.include "ldma.s"
	.include "ol_sprite.s"
