#ifndef _SPRITE2_H_
#define _SPRITE2_H_

#include "queue.h"
//#include "macros.h"

/*
 * Copyright (c) 2001-2002 VM Labs, Inc.  All rights reserved.
 *
 * NOTICE: VM Labs permits you to use, modify, and distribute this file
 * in accordance with the terms of the VM Labs license agreement
 * accompanying it. If you have received this file from a source other
 * than VM Labs, then your use, modification, or distribution of it
 * requires the prior written permission of VM Labs.
*/

#ifdef LIBSPRITE_INLINE
#define INLINE1 static inline
#define INLINE2 __attribute__((always_inline))
#else
#define INLINE1
#define INLINE2
#endif

#if 0
#ifndef LIBSPRITE_INLINE
typedef struct priqueue;

typedef struct priqueue {
  //  qPtr prev;
  //  qPtr next;
  struct priqueue *prev;
  struct priqueue *next;
  unsigned long priority;
} PriorityQueueMember;

// here's the queue as a whole
typedef struct pqueue {
    PriorityQueueMember *list;
} PriorityQueue;

void *         QInitMem(void *mem, unsigned num_elems, unsigned size_elems);
void           QInit_func(PriorityQueue *queue, PriorityQueueMember *initialmem);

void           QAdd(PriorityQueue *queue, PriorityQueueMember *member, int comparison);
int            QRemove(PriorityQueue *queue, PriorityQueueMember *member);
PriorityQueueMember * QGetHead(PriorityQueue *queue);
void           QPutHead(PriorityQueue *queue, PriorityQueueMember *m);
int            QIsEmpty(PriorityQueue *q);

#define QInit(queue, initial)\
    ((PriorityQueue *) queue)->list = initial;
#endif
#endif

/* priority macros */
#define ComparePriorities(a,b) ((long)((a) - (b)))
#define LowerPriority(a,b) (ComparePriorities(a,b) > 0)
#define LowerOrEqualPriority(a,b) (ComparePriorities(a,b) >= 0)
#define HigherPriority(a,b) (ComparePriorities(a,b) < 0)
#define HigherOrEqualPriority(a,b) (ComparePriorities(a,b) <= 0)

#define NOPACK

#ifdef NOPACK
        #define PACKED
#else

#ifdef __GNUC__
	#define PACKED __attribute__ ((packed))
#else
	#define PACKED
#endif // __GNUC__

#endif // NOPACK

typedef struct
{
	void *img;
	long dmaflags;
	long width, height;
} PACKED SPR_IMAGE_INFO;

typedef struct
{
  long dmaflags;
  void *dest;
  short clip_max_x;
  short clip_min_x;
  short clip_max_y;
  short clip_min_y;
  unsigned long renderer;
  int clrParam1;
  int clrParam2;
  int clrParam3;
} PACKED RASTER_INFO;

enum SpriteTypes
{
	kSpriteSimple            = 0,
	kSpriteYar               = kSpriteSimple,
	kSpriteSimpleTrans       = 1,
	kSpriteInterpolatedTrans = 2,
	kSpriteInterpolated      = 3,
	
	kSpriteYarTp,  // 4
	kSpriteYarTpBrightSimple,
	kSpriteYarTpBright,
	kSpriteYarTpBrightAddRGB,
	
	kSpriteYarTlNoSat,  // 8
	kSpriteYarTlNoSatAlpha,
	kSpriteYarTlNoSatRGBAlpha,
	kSpriteYarTlNoSatAddRGBAlpha,
	kSpriteYarTlNoSatRGBAddRGBAlpha,
	
	kSpriteYarTlSatEnd,  // 13
	kSpriteYarTlSatEndRGB,
	kSpriteYarTlSatEndAlpha,
	
	kSpriteYarTlSatMidEnd,  // 16
	kSpriteYarTlSatMidEndAlpha,
	
	kSpriteYarTlSatZero,  // 18
	kSpriteYarTlSatZero_RGB,
	kSpriteYarTlSatEndZero,
	kSpriteYarTlSatEndZeroAlpha,
	
	kSpriteYarAll,

	kSpritePoint,                    // renderer dedicated to drawing single pixel points
	kSpriteLine,                     // renderer dedicated to drawing lines
	kSpriteGLine,                    // renderer dedicated to drawing gradiated lines

	kClearColor,                     // clear renderer to blank to a constant color
	kClearCopyFade,                  // clear renderer to copy from and fade another buffer
	kClearCopyTile,                  // clear by copying from another buffer, tiling if required
	kClearStars,                     // clears to a color and adds single pixel stars

	kSpriteSimpleStart,
	kSpriteSimpleTp = kSpriteSimpleStart,  // simple sprite (no rot or scale) with transparency
	kSpriteSimpleEnd,

	kSpriteRendererMax               // will get its value as one more than previous
};

typedef struct {
  PriorityQueue *parent;
  PriorityQueueMember q;
  long rendererPtr;   // Renderer info ([31:28] = size, [27:0] = addr)
  short x,y;          // Dest position
  short w,h;          // sprite width and height
  long src_x,src_y;   // source offset (16:16)
  long xscale,yscale; // X and Y scale (16:16)
  long angle;         // rotation angle (16.16) 0.0 = 0 deg, 1.0 = 360 deg
  long trans;         // Translucency/Mix
  long dmaflags;      // dmaflags source (READ BIT!!!)
  void *source;       // source image
  long transColor;    // transparent pixel value
  long addRGB;        // amount of RGB to add to each sprite (R[31:24], G[23:16], B[15:8])
  long tint;          // tint color (31:8) & hint bits (7:0)
} PACKED SPRITE;

#define UNUSED   0xFFFFFFFF
#define UNUSED16 0xFFFF

#define MAX_POINTS 12

typedef struct {
  PriorityQueue *parent;
  PriorityQueueMember q;
  long rendererPtr;                 // Renderer info ([31:28] = size, [27:0] = addr)
  unsigned long coord[MAX_POINTS];  // Can hold up to twelve points per structure
  long color;                       // Color of these points
} PACKED sPoint;

#define MAX_LINES 4

typedef struct {
  unsigned long majorCoords;
  unsigned long minorCoords;
  unsigned long dMinor_dMajor;  // bit 0 also holds a flag to tell whether to swap major/minor
} sLineType;

typedef struct {
  PriorityQueue *parent;
  PriorityQueueMember q;
  long rendererPtr;            // Renderer info ([31:28] = size, [27:0] = addr)
  sLineType coord[MAX_LINES];  // Can hold up to four lines per structure
  long color;                  // Color of these lines
} PACKED sLine;

#define MAX_GLINES 1

typedef struct {
  unsigned long majorCoords;
  unsigned long minorCoords;
  unsigned long dMinor_dMajor;  // bit 0 also holds a flag to tell whether to swap major/minor
  unsigned long colorStart;     // color at start point
  unsigned long colorStepY;     // color increment per major step Y
  unsigned long colorStepCr;    // color increment per major step Cr
  unsigned long colorStepCb;    // color increment per major step Cb
  unsigned long reserved[5];
} sGLineType;

typedef struct {
  PriorityQueue *parent;
  PriorityQueueMember q;
  long rendererPtr;             // Renderer info ([31:28] = size, [27:0] = addr)
  sGLineType data[MAX_GLINES];  // Can hold only one gline per structure
  long hint;                    // Sprite hints/flags (tint is covered in the sGLineType structure)
} PACKED sGLine;

typedef struct {
  //  short x, y;
  short y, x;
} sSprCoord;

#define MAX_SPR_SIMPLE 4

typedef struct {
  PriorityQueue *parent;
  PriorityQueueMember q;
  long rendererPtr;   // Renderer info ([31:28] = size, [27:0] = addr)
  short x,y;          // Dest position
  short w,h;          // sprite width and height
  long src_x,src_y;   // source offset (16:16)
  sSprCoord aSprCoord[MAX_SPR_SIMPLE];        // Dest positions
  long dmaflags;      // dmaflags source (READ BIT!!!)
  void *source;       // source image
  long reserved1;
  long reserved2;
  long tint;          // tint color (31:8) & hint bits (7:0)
} PACKED sSpriteSimple;

typedef struct {
  short x,y;
  long color;
} PACKED sClearStar;

#define MAX_BOXES 3

typedef struct {
  short y1, x1;                 // top-left coordinate
  short y2, x2;                 // bottom-right coordinate
  unsigned long color;          // has box translucency factored in already
  unsigned long transBG;        // amount to scale the background color
} sBoxType;

typedef struct {
  PriorityQueue *parent;
  PriorityQueueMember q;
  long rendererPtr;             // Renderer info ([31:28] = size, [27:0] = addr)
  sBoxType data[MAX_BOXES];     // Can hold four box structures
  long hint;                    // Sprite hints/flags (tint is covered in the sBoxType structure)
} PACKED sBox;

void    SPRFill(int initx, int inity, int wide, int high, int color);
int     SPRInit(unsigned long mask, int sliceHeight, int flags);
int     SPRReset();
void    SPRRemoveStickySprite( SPRITE *spr, int displayListIndex );
void    SPRClearStickySprites( int displayListIndex );
int     SPRClearDisplayList();
int     SPRSetDestScreen(void *scr, long dmaflags, int minx, int miny, int maxx, int maxy,
			 int bgColor);
int     SPRInstallTGAImage(char *tga, int mode, SPR_IMAGE_INFO *info, int transColorRGB);
int     SPRSetSourceImage(char *scr, int dmafl, int width, int height, SPR_IMAGE_INFO *info);
INLINE1 SPRITE* SPRCreateSprite(SPR_IMAGE_INFO *info,int x, int y, int w, int h);// INLINE2;
SPRITE* SPRCloneSprite(SPRITE *s);
SPRITE* SPRAddPoint(int x, int y, int color, int depth, int displayListIndex);
SPRITE* SPRAddLine( int x0, int y0, int x1, int y1, unsigned int color1, unsigned int color2,
		    int depth, int displayListIndex, int flags );
int     SPRAddSprite(SPRITE * sprite, int x, int y, int angle, int xscale, int yscale,
		     int type, int tc, int depth);
INLINE1 int     SPRAddSpriteYaroze(SPRITE *sprite, int x, int y, int angle, int xscale, int yscale, 
			   int transFG, int transBG, int scaleR, int scaleG, int scaleB,
			   unsigned long addRGB, int depth, int hint, int displayListIndex);
int     SPRModifySprite(SPRITE * sprite, int x, int y, int angle, int xscale, int yscale,
			int type, int tc, int depth);
SPRITE* SPRCreateAndAddSprite(SPR_IMAGE_INFO *info, int srcX, int srcY, int srcW, int srcH,
			      int x, int y, int angle, int xscale, int yscale, int type,
			      int tc, int depth);
int     SPRRemoveSprite         (SPRITE *sprite);
int     SPRDeleteSprite         (SPRITE *sprite);
int     SPRRemoveAndDeleteSprite(SPRITE *sprite);
void    SPRSetSpriteSource      (SPRITE * sprite, int x, int y, int w, int h);
void    SPRSetImageInfo         (SPRITE *sprite, SPR_IMAGE_INFO *info );
void    SPRSetSpriteXY          (SPRITE * sprite, int x, int y);
void    SPRSetSpriteScale       (SPRITE * sprite, int xscale, int yscale);
void    SPRSetSpriteRotation    (SPRITE * sprite, int angle);
void    SPRSetSpriteType        (SPRITE * sprite, int type);
void    SPRSetSpriteTColor      (SPRITE * sprite, int tc);
void    SPRSetSpriteDepth       (SPRITE * sprite, int depth);

void    SPRDraw(int clrScreen, int wait, int displayListIndex);
int     SPRWait(void);
int     SPRBlitter(char *src, long src_dmaflags, int src_x, int src_y, int src_w, int src_h,
		   char *dest, long dest_dmaflags, int dest_x, int dest_y,
		   int scalex, int scaley, int angle,
		   int clip_min_x,int clip_min_y,int clip_max_x,int clip_max_y,
		   int type, int transColor);
int SPRDebug(int mode, char *str);
void SPRPrintRendererCounters();
void SPRResetRendererCounters();

// Debug
void SPRPrintAndClearDelay();
void SPRSetOffset(int x, int y);
int  SPRCountAllElements(int mode, char *str1, char *str2);

#define SPR_FLAG_ADD_REVERSE   (1<<0)
#define SPR_FLAG_DOUBLE_BUFFER (1<<1)
#define SPR_FLAG_CLEAR_FAST    (1<<2)
#define SPR_FLAG_USE_ALPHA     (1<<3)

#define kSprHintNoTransp       (1<<0)
#define kSprHintMatPrepOne     (1<<1)
#define kSprHintMatPrepBoth    (1<<2)
#define kSprHintPrint          (1<<3)
#define kSprHintDebug          (1<<4)
#define kSprHintRenderImm      (1<<5)
#define kSprHintBakeRGB        (1<<7)
#define kSprHintBakeYCrCb      (1<<13)
#define kSprHintRenderer       (1<<14)
#define kSprHintSticky         (1<<15)

extern int libspritePrintSprites;

typedef struct {
  long usecPrev;
  int cur;
  int worst;
  int cum;
  int count;
} sSprPerfMon;

#define MAX_PERF_MON 4
#define TIME_ELAPSED_DELAY 30
extern sSprPerfMon aSprPerfMon[MAX_PERF_MON];

#define MAX_DISPLAY_LISTS 8

#endif
